"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AmplifyFormRenderer = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const primitive_1 = require("../primitive");
const customComponent_1 = __importDefault(require("./customComponent"));
const form_1 = __importDefault(require("./form"));
const react_component_renderer_1 = require("../react-component-renderer");
const forms_1 = require("../forms");
class AmplifyFormRenderer extends forms_1.ReactFormTemplateRenderer {
    renderJsx(formComponent, parent) {
        const node = new codegen_ui_1.StudioNode(formComponent, parent);
        const renderChildren = (children) => children.map((child) => this.renderJsx(child, node));
        // add Primitive in alphabetical order
        switch (formComponent.componentType) {
            case primitive_1.Primitive.Alert:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Badge:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Button:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.ButtonGroup:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Card:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.CheckboxField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case 'form':
                return new form_1.default(formComponent, 
                // this component is the current form
                this.component, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case 'option':
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Divider:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement();
            case primitive_1.Primitive.Expander:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.ExpanderItem:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Flex:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Grid:
                if (!parent) {
                    return new form_1.default(formComponent, 
                    // this component is the current form
                    this.component, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
                }
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Heading:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Icon:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Image:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement();
            case primitive_1.Primitive.Link:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Loader:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.MenuButton:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.MenuItem:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Menu:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Pagination:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.PasswordField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.PhoneNumberField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Placeholder:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Radio:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.RadioGroupField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Rating:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.ScrollView:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.SearchField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.SelectField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.SliderField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement();
            case primitive_1.Primitive.StepperField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.SwitchField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.TabItem:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Tabs:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Table:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.TableBody:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.TableCell:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.TableFoot:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.TableHead:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.TableRow:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Text:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.TextAreaField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.TextField:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.ToggleButton:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.ToggleButtonGroup:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.View:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.VisuallyHidden:
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            case primitive_1.Primitive.Autocomplete:
                // TODO: after AmplifyUI implements Autocomplete, import Props type
                return new react_component_renderer_1.ReactComponentRenderer(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
            default:
                return new customComponent_1.default(formComponent, this.componentMetadata, this.importCollection, parent).renderElement(renderChildren);
        }
    }
}
exports.AmplifyFormRenderer = AmplifyFormRenderer;
//# sourceMappingURL=amplify-form-renderer.js.map