"use strict";
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AmplifyViewRenderer = void 0;
const typescript_1 = require("typescript");
const react_view_renderer_1 = require("../views/react-view-renderer");
const primitive_1 = require("../primitive");
const react_table_renderer_1 = require("../react-table-renderer");
const react_expander_renderer_1 = require("../react-expander-renderer");
class AmplifyViewRenderer extends react_view_renderer_1.ReactViewTemplateRenderer {
    renderJsx() {
        switch (this.viewComponent.viewConfiguration.type) {
            case primitive_1.Primitive.Table:
                return new react_table_renderer_1.ReactTableRenderer(this.viewComponent, this.viewDefinition, this.viewMetadata, this.importCollection).renderElement();
            case 'Collection':
                switch (this.viewComponent.viewConfiguration.collection.collectionType) {
                    case 'expander':
                        return new react_expander_renderer_1.ReactExpanderRenderer(this.viewComponent, this.importCollection).renderElement();
                    default:
                        throw new Error(`Collection type ${this.viewComponent.viewConfiguration.collection.collectionType} is not supported`);
                }
            default:
                return typescript_1.factory.createJsxFragment(typescript_1.factory.createJsxOpeningFragment(), [], typescript_1.factory.createJsxJsxClosingFragment());
        }
    }
}
exports.AmplifyViewRenderer = AmplifyViewRenderer;
//# sourceMappingURL=amplify-view-renderer.js.map