"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFormattedValueExpression = exports.buildTargetVariable = void 0;
const typescript_1 = require("typescript");
const form_state_1 = require("./form-state");
/*
Builds the event target variable. Example:
let { value } = e.target;
*/
const expressionMap = {
    // e
    e: typescript_1.factory.createIdentifier('e'),
    // value
    value: typescript_1.factory.createIdentifier('value'),
    // e.target.checked
    eTargetChecked: typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('e'), typescript_1.factory.createIdentifier('target')), typescript_1.factory.createIdentifier('checked')),
    // e.target
    eTarget: typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('e'), typescript_1.factory.createIdentifier('target')),
    // {value}
    destructuredValue: typescript_1.factory.createObjectBindingPattern([
        typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('value'), undefined),
    ]),
    // e.target.value
    eTargetValue: typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('e'), typescript_1.factory.createIdentifier('target')), typescript_1.factory.createIdentifier('value')),
};
// default variable statement
const setVariableStatement = (lhs, assignment) => typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([typescript_1.factory.createVariableDeclaration(lhs, undefined, undefined, assignment)], typescript_1.NodeFlags.Let));
const numberVariableStatement = (fieldName, variable) => {
    return [
        variable,
        typescript_1.factory.createIfStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('isNaN'), undefined, [typescript_1.factory.createIdentifier('value')]), typescript_1.factory.createBlock([
            (0, form_state_1.setErrorState)(typescript_1.factory.createIdentifier(fieldName), typescript_1.factory.createStringLiteral('Value must be a valid number')),
            typescript_1.factory.createReturnStatement(undefined),
        ], true), undefined),
    ];
};
/**
 * const date = new Date(e.target.value);
 *
 * if(!(date instanceof Date && !isNaN(date))) {
 *
 * setErrors((errors) => ({ ...errors, [key]: 'The value must be a valid date' }));
 *
 * return
 *
 * }
 *
 * let value = Number(value);
 *
 * @param fieldName name of field used to set error if date is invalid
 * @returns return expression block {example above}
 */
const timestampVariableStatement = (fieldName) => {
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('date'), undefined, undefined, typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Date'), undefined, [
                (0, form_state_1.buildAccessChain)(['e', 'target', 'value'], false),
            ])),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createIfStatement(typescript_1.factory.createPrefixUnaryExpression(typescript_1.SyntaxKind.ExclamationToken, typescript_1.factory.createParenthesizedExpression(typescript_1.factory.createBinaryExpression(typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('date'), typescript_1.factory.createToken(typescript_1.SyntaxKind.InstanceOfKeyword), typescript_1.factory.createIdentifier('Date')), typescript_1.factory.createToken(typescript_1.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createPrefixUnaryExpression(typescript_1.SyntaxKind.ExclamationToken, typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('isNaN'), undefined, [
            typescript_1.factory.createIdentifier('date'),
        ]))))), typescript_1.factory.createBlock([
            (0, form_state_1.setErrorState)(typescript_1.factory.createIdentifier(fieldName), typescript_1.factory.createStringLiteral('The value must be a valid date')),
            typescript_1.factory.createReturnStatement(undefined),
        ], false)),
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(expressionMap.value, undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('Number'), undefined, [
                typescript_1.factory.createIdentifier('date'),
            ])),
        ], typescript_1.NodeFlags.Let)),
    ];
};
const buildTargetVariable = (fieldType, fieldName, dataType) => {
    var _a, _b, _c, _d;
    const fieldTypeToExpressionMap = {
        SliderField: {
            expression: expressionMap.e,
            identifier: expressionMap.value,
        },
        StepperField: {
            expression: expressionMap.e,
            identifier: expressionMap.value,
        },
        SwitchField: {
            expression: expressionMap.eTargetChecked,
            identifier: expressionMap.value,
        },
        CheckboxField: {
            expression: expressionMap.eTargetChecked,
            identifier: expressionMap.value,
        },
        ToggleButton: {
            expression: typescript_1.factory.createPrefixUnaryExpression(typescript_1.SyntaxKind.ExclamationToken, typescript_1.factory.createIdentifier(fieldName)),
            identifier: expressionMap.value,
        },
    };
    let expression = (_b = (_a = fieldTypeToExpressionMap[fieldType]) === null || _a === void 0 ? void 0 : _a.expression) !== null && _b !== void 0 ? _b : expressionMap.eTarget;
    let defaultIdentifier = (_d = (_c = fieldTypeToExpressionMap[fieldType]) === null || _c === void 0 ? void 0 : _c.identifier) !== null && _d !== void 0 ? _d : expressionMap.destructuredValue;
    switch (dataType) {
        case 'AWSTimestamp':
            // validate date first then cast to number
            return timestampVariableStatement(fieldName);
        case 'Float':
            if (fieldType === 'TextField') {
                // value = Number(e.target.value);
                defaultIdentifier = expressionMap.value;
                expression = typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('Number'), undefined, [
                    expressionMap.eTargetValue,
                ]);
            }
            return numberVariableStatement(fieldName, setVariableStatement(defaultIdentifier, expression));
            break;
        case 'Int':
            if (fieldType === 'TextField') {
                // value = parseInt(e.target.value);
                defaultIdentifier = expressionMap.value;
                expression = typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('parseInt'), undefined, [
                    expressionMap.eTargetValue,
                ]);
            }
            return numberVariableStatement(fieldName, setVariableStatement(defaultIdentifier, expression));
        case 'Boolean':
            if (fieldType === 'RadioGroupField') {
                // value = e.target.value === 'true'
                defaultIdentifier = expressionMap.value;
                expression = typescript_1.factory.createBinaryExpression(expressionMap.eTargetValue, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsEqualsEqualsToken), typescript_1.factory.createStringLiteral('true'));
            }
            return [setVariableStatement(defaultIdentifier, expression)];
        default:
            return [setVariableStatement(defaultIdentifier, expression)];
    }
};
exports.buildTargetVariable = buildTargetVariable;
const getFormattedValueExpression = (dataType) => {
    // setStateExpression(getCurrentValueName(renderedFieldName), factory.createIdentifier('value')),
    let setExpression = typescript_1.factory.createIdentifier('value');
    if (dataType === 'AWSDateTime') {
        setExpression = typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Date'), undefined, [typescript_1.factory.createIdentifier('value')]), typescript_1.factory.createIdentifier('toISOString')), undefined, []);
    }
    return setExpression;
};
exports.getFormattedValueExpression = getFormattedValueExpression;
//# sourceMappingURL=event-targets.js.map