"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _ImportCollection_collection;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportCollection = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const typescript_1 = require("typescript");
const path_1 = __importDefault(require("path"));
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const import_mapping_1 = require("./import-mapping");
const primitive_1 = require("../primitive");
const helpers_1 = require("../helpers");
class ImportCollection {
    constructor(componentMetadata) {
        _ImportCollection_collection.set(this, new Map());
        this.importAlias = new Map();
        this.importedNames = new Set(Object.values((componentMetadata === null || componentMetadata === void 0 ? void 0 : componentMetadata.componentNameToTypeMap) || {}).concat(codegen_ui_1.reservedWords));
        // Add form fields so we dont reuse the identifier
        if (componentMetadata === null || componentMetadata === void 0 ? void 0 : componentMetadata.formMetadata) {
            Object.keys(componentMetadata.formMetadata.fieldConfigs).forEach((key) => this.importedNames.add(key));
        }
    }
    addMappedImport(importValue) {
        const importPackage = import_mapping_1.ImportMapping[importValue];
        this.addImport(importPackage, importValue);
    }
    addImport(packageName, importName) {
        if (!__classPrivateFieldGet(this, _ImportCollection_collection, "f").has(packageName)) {
            __classPrivateFieldGet(this, _ImportCollection_collection, "f").set(packageName, new Set());
        }
        const existingPackage = __classPrivateFieldGet(this, _ImportCollection_collection, "f").get(packageName);
        if (!(existingPackage === null || existingPackage === void 0 ? void 0 : existingPackage.has(importName))) {
            existingPackage === null || existingPackage === void 0 ? void 0 : existingPackage.add(importName);
            if (packageName !== import_mapping_1.ImportSource.LOCAL_MODELS) {
                this.importedNames.add(importName);
            }
        }
        if (packageName === import_mapping_1.ImportSource.LOCAL_MODELS) {
            const existingPackageAlias = this.importAlias.get(packageName);
            const existingAlias = existingPackageAlias === null || existingPackageAlias === void 0 ? void 0 : existingPackageAlias.get(importName);
            if (existingAlias)
                return existingAlias;
            const modelAlias = (0, helpers_1.createUniqueName)(importName, (input) => this.importedNames.has(input) || (input.endsWith('Props') && (0, primitive_1.isPrimitive)(input.replace(/Props/g, ''))));
            if (existingPackageAlias) {
                existingPackageAlias.set(importName, modelAlias);
            }
            else {
                const aliasMap = new Map();
                aliasMap.set(importName, modelAlias);
                this.importAlias.set(packageName, aliasMap);
            }
            this.importedNames.add(modelAlias);
            return modelAlias;
        }
        return importName;
    }
    removeImportSource(packageImport) {
        __classPrivateFieldGet(this, _ImportCollection_collection, "f").delete(packageImport);
    }
    getMappedAlias(packageName, importName) {
        var _a;
        return ((_a = this.importAlias.get(packageName)) === null || _a === void 0 ? void 0 : _a.get(importName)) || importName;
    }
    mergeCollections(otherCollection) {
        __classPrivateFieldGet(otherCollection, _ImportCollection_collection, "f").forEach((value, key) => {
            [...value].forEach((singlePackage) => {
                this.addImport(key, singlePackage);
            });
        });
    }
    buildSampleSnippetImports(topComponentName) {
        return [
            typescript_1.factory.createImportDeclaration(undefined, undefined, typescript_1.factory.createImportClause(false, undefined, typescript_1.factory.createNamedImports([
                typescript_1.factory.createImportSpecifier(undefined, typescript_1.factory.createIdentifier(topComponentName)),
            ])), typescript_1.factory.createStringLiteral('./ui-components')),
        ];
    }
    buildImportStatements(skipReactImport) {
        const importDeclarations = []
            .concat(skipReactImport
            ? []
            : [
                typescript_1.factory.createImportDeclaration(undefined, undefined, typescript_1.factory.createImportClause(false, undefined, typescript_1.factory.createNamespaceImport(typescript_1.factory.createIdentifier('React'))), typescript_1.factory.createStringLiteral('react')),
            ])
            .concat(Array.from(__classPrivateFieldGet(this, _ImportCollection_collection, "f")).map(([moduleName, imports]) => {
            const namedImports = [...imports].filter((namedImport) => namedImport !== 'default').sort();
            const aliasMap = this.importAlias.get(moduleName);
            if (aliasMap) {
                const importClause = typescript_1.factory.createImportClause(false, undefined, typescript_1.factory.createNamedImports([...imports].map((item) => {
                    const alias = aliasMap.get(item);
                    return typescript_1.factory.createImportSpecifier(alias && alias !== item ? typescript_1.factory.createIdentifier(item) : undefined, typescript_1.factory.createIdentifier(alias !== null && alias !== void 0 ? alias : item));
                })));
                return typescript_1.factory.createImportDeclaration(undefined, undefined, importClause, typescript_1.factory.createStringLiteral(moduleName));
            }
            return typescript_1.factory.createImportDeclaration(undefined, undefined, typescript_1.factory.createImportClause(false, 
            // use module name as default import name
            [...imports].indexOf('default') >= 0 ? typescript_1.factory.createIdentifier(path_1.default.basename(moduleName)) : undefined, typescript_1.factory.createNamedImports(namedImports.map((item) => {
                return typescript_1.factory.createImportSpecifier(undefined, typescript_1.factory.createIdentifier(item));
            }))), typescript_1.factory.createStringLiteral(moduleName));
        }));
        return importDeclarations;
    }
}
exports.ImportCollection = ImportCollection;
_ImportCollection_collection = new WeakMap();
//# sourceMappingURL=import-collection.js.map