"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactUtilsStudioTemplateRenderer = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const os_1 = require("os");
const typescript_1 = require("typescript");
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const react_render_config_1 = require("./react-render-config");
const imports_1 = require("./imports");
const react_output_manager_1 = require("./react-output-manager");
const react_studio_template_renderer_helper_1 = require("./react-studio-template-renderer-helper");
const validation_1 = require("./utils/forms/validation");
const json_path_fetch_1 = require("./utils/json-path-fetch");
const string_formatter_1 = require("./utils/string-formatter");
class ReactUtilsStudioTemplateRenderer extends codegen_ui_1.StudioTemplateRenderer {
    constructor(utils, renderConfig) {
        super(utils, new react_output_manager_1.ReactOutputManager(), renderConfig);
        this.importCollection = new imports_1.ImportCollection();
        this.utils = utils;
        this.renderConfig = {
            ...react_studio_template_renderer_helper_1.defaultRenderConfig,
            ...renderConfig,
            renderTypeDeclarations: false, // Never render type declarations for index.js|ts file.
        };
        this.fileName = `utils.${(0, react_render_config_1.scriptKindToFileExtensionNonReact)(this.renderConfig.script)}`;
    }
    renderComponentInternal() {
        const { printer, file } = (0, react_studio_template_renderer_helper_1.buildPrinter)(this.fileName, this.renderConfig);
        const utilsStatements = [];
        const skipReactImport = true;
        const utilsSet = new Set(this.utils);
        if (utilsSet.has('validation')) {
            utilsStatements.push(...(0, validation_1.generateValidationFunction)());
        }
        if (utilsSet.has('formatter')) {
            utilsStatements.push(...(0, string_formatter_1.generateFormatUtil)());
        }
        if (utilsSet.has('fetchByPath')) {
            utilsStatements.push((0, json_path_fetch_1.getFetchByPathNodeFunction)());
        }
        let componentText = `/* eslint-disable */${os_1.EOL}`;
        const imports = this.importCollection.buildImportStatements(skipReactImport);
        imports.forEach((importStatement) => {
            const result = printer.printNode(typescript_1.EmitHint.Unspecified, importStatement, file);
            componentText += result + os_1.EOL;
        });
        componentText += os_1.EOL;
        utilsStatements.forEach((util) => {
            const result = printer.printNode(typescript_1.EmitHint.Unspecified, util, file);
            componentText += result + os_1.EOL;
        });
        componentText += os_1.EOL;
        const { componentText: transpliedText } = (0, react_studio_template_renderer_helper_1.transpile)(componentText, this.renderConfig);
        return {
            componentText: transpliedText,
            renderComponentToFilesystem: async (outputPath) => {
                await this.renderComponentToFilesystem(transpliedText)(this.fileName)(outputPath);
            },
        };
    }
    // no-op
    validateSchema() { }
}
exports.ReactUtilsStudioTemplateRenderer = ReactUtilsStudioTemplateRenderer;
//# sourceMappingURL=react-utils-studio-template-renderer.js.map