"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFetchByPathNodeFunction = exports.fetchByPath = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const typescript_1 = require("typescript");
/**
 * does not support array types within objects as it's currently not supported
 *
 * ref: https://stackoverflow.com/questions/45942118/lodash-return-array-of-values-if-the-path-is-valid
 *
 * @param input object input
 * @param path dot notation path for the provided input
 * @param accumlator array
 * @returns returns value at the end of object
 */
const fetchByPath = (input, path, accumlator = []) => {
    const currentPath = path.split('.');
    const head = currentPath.shift();
    if (input && head && input[head] !== undefined) {
        if (!currentPath.length) {
            accumlator.push(input[head]);
        }
        else {
            (0, exports.fetchByPath)(input[head], currentPath.join('.'), accumlator);
        }
    }
    return accumlator[0];
};
exports.fetchByPath = fetchByPath;
/**
 * get the generated output of the fetchByPath function in TS AST
 *
 * @returns VariableStatement
 */
const getFetchByPathNodeFunction = () => {
    return typescript_1.factory.createVariableStatement([typescript_1.factory.createModifier(typescript_1.SyntaxKind.ExportKeyword)], typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('fetchByPath'), undefined, undefined, typescript_1.factory.createArrowFunction(undefined, [
            typescript_1.factory.createTypeParameterDeclaration(typescript_1.factory.createIdentifier('T'), typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('Record'), [
                typescript_1.factory.createKeywordTypeNode(typescript_1.SyntaxKind.StringKeyword),
                typescript_1.factory.createKeywordTypeNode(typescript_1.SyntaxKind.AnyKeyword),
            ]), undefined),
        ], [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('input'), undefined, typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('T'), undefined), undefined),
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('path'), undefined, undefined, typescript_1.factory.createStringLiteral('')),
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('accumlator'), undefined, typescript_1.factory.createArrayTypeNode(typescript_1.factory.createKeywordTypeNode(typescript_1.SyntaxKind.AnyKeyword)), typescript_1.factory.createArrayLiteralExpression([], false)),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('currentPath'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('path'), typescript_1.factory.createIdentifier('split')), undefined, [typescript_1.factory.createStringLiteral('.')])),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('head'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('currentPath'), typescript_1.factory.createIdentifier('shift')), undefined, [])),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createIfStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('input'), typescript_1.factory.createToken(typescript_1.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createIdentifier('head')), typescript_1.factory.createToken(typescript_1.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createBinaryExpression(typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('input'), typescript_1.factory.createIdentifier('head')), typescript_1.factory.createToken(typescript_1.SyntaxKind.ExclamationEqualsEqualsToken), typescript_1.factory.createIdentifier('undefined'))), typescript_1.factory.createBlock([
                typescript_1.factory.createIfStatement(typescript_1.factory.createPrefixUnaryExpression(typescript_1.SyntaxKind.ExclamationToken, typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('currentPath'), typescript_1.factory.createIdentifier('length'))), typescript_1.factory.createBlock([
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('accumlator'), typescript_1.factory.createIdentifier('push')), undefined, [
                        typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('input'), typescript_1.factory.createIdentifier('head')),
                    ])),
                ], true), typescript_1.factory.createBlock([
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('fetchByPath'), undefined, [
                        typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('input'), typescript_1.factory.createIdentifier('head')),
                        typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('currentPath'), typescript_1.factory.createIdentifier('join')), undefined, [typescript_1.factory.createStringLiteral('.')]),
                        typescript_1.factory.createIdentifier('accumlator'),
                    ])),
                ], true)),
            ], true), undefined),
            typescript_1.factory.createReturnStatement(typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('accumlator'), typescript_1.factory.createNumericLiteral('0'))),
        ], true))),
    ], typescript_1.NodeFlags.Const));
};
exports.getFetchByPathNodeFunction = getFetchByPathNodeFunction;
//# sourceMappingURL=json-path-fetch.js.map