"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapGenericEventToReact = exports.buildActionEvent = exports.buildBindingEvent = exports.buildOpeningElementEvents = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const typescript_1 = require("typescript");
const action_1 = require("./action");
const react_component_render_helper_1 = require("../react-component-render-helper");
const primitive_1 = require("../primitive");
/*
 * Temporary hardcoded mapping of generic to react events, long-term this will be exported by amplify-ui.
 */
const genericEventToReactEventMapping = {
    [codegen_ui_1.StudioGenericEvent.onClick]: 'onClick',
    [codegen_ui_1.StudioGenericEvent.onDoubleClick]: 'onDoubleClick',
    [codegen_ui_1.StudioGenericEvent.onMouseDown]: 'onMouseDown',
    [codegen_ui_1.StudioGenericEvent.onMouseEnter]: 'onMouseEnter',
    [codegen_ui_1.StudioGenericEvent.onMouseLeave]: 'onMouseLeave',
    [codegen_ui_1.StudioGenericEvent.onMouseMove]: 'onMouseMove',
    [codegen_ui_1.StudioGenericEvent.onMouseOut]: 'onMouseOut',
    [codegen_ui_1.StudioGenericEvent.onMouseOver]: 'onMouseOver',
    [codegen_ui_1.StudioGenericEvent.onMouseUp]: 'onMouseUp',
    [codegen_ui_1.StudioGenericEvent.onChange]: 'onChange',
    [codegen_ui_1.StudioGenericEvent.onInput]: 'onInput',
    [codegen_ui_1.StudioGenericEvent.onFocus]: 'onFocus',
    [codegen_ui_1.StudioGenericEvent.onBlur]: 'onBlur',
    [codegen_ui_1.StudioGenericEvent.onKeyDown]: 'onKeyDown',
    [codegen_ui_1.StudioGenericEvent.onKeyPress]: 'onKeyPress',
    [codegen_ui_1.StudioGenericEvent.onKeyUp]: 'onKeyUp',
    [codegen_ui_1.StudioGenericEvent.onSubmit]: 'onSubmit',
};
const genericEventToReactEventMappingOverrides = {
    [primitive_1.Primitive.StepperField]: {
        [codegen_ui_1.StudioGenericEvent.onChange]: 'onStepChange',
    },
};
function buildOpeningElementEvents(componentType, event, name, componentName) {
    if ((0, react_component_render_helper_1.isBoundEvent)(event)) {
        return buildBindingEvent(componentType, event, name);
    }
    if ((0, react_component_render_helper_1.isActionEvent)(event)) {
        return buildActionEvent(componentType, name, componentName);
    }
    return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier(name), undefined);
}
exports.buildOpeningElementEvents = buildOpeningElementEvents;
function buildBindingEvent(componentType, event, eventName) {
    const expr = typescript_1.factory.createIdentifier(event.bindingEvent);
    return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier(mapGenericEventToReact(componentType, eventName)), typescript_1.factory.createJsxExpression(undefined, expr));
}
exports.buildBindingEvent = buildBindingEvent;
function buildActionEvent(componentType, eventName, componentName) {
    return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier(mapGenericEventToReact(componentType, eventName)), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createArrowFunction(undefined, undefined, [], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier((0, action_1.getActionIdentifier)(componentName, eventName)), undefined, [])),
    ], false))));
}
exports.buildActionEvent = buildActionEvent;
function mapGenericEventToReact(componentType, genericEventBinding) {
    const componentOverrides = genericEventToReactEventMappingOverrides[componentType];
    if (componentOverrides && componentOverrides[genericEventBinding]) {
        return componentOverrides[genericEventBinding];
    }
    const reactEventMapping = genericEventToReactEventMapping[genericEventBinding];
    if (!reactEventMapping) {
        throw new Error(`${genericEventBinding} is not a possible event.`);
    }
    return reactEventMapping;
}
exports.mapGenericEventToReact = mapGenericEventToReact;
//# sourceMappingURL=events.js.map